#!/usr/local/bin/perl

# this program generates the c src code for getSisThickness.c
# using the template file getSisThickness.src which
# contains all the c code, plus many tags which will be replaced by this
# program.  it is done this way because the tags will be replaced
# with a large quantitiy of information taken from the databases affiliated
# with the getSisThickness program.  the databases are much more
# maintainable in their current form than they would be if they were only
# held in the c source file.  thus the separation and generation.

# IMPORTANT!!!! the interferometer map databases provided by mew in
# early 1997 were off by a factor of 2.  THIS PROGRAM CORRECTS THE
# DATA BEFORE IT IS PLACED INTO THE ARRAYS IN getSisThickness.c.
# THIS PROGRAM ALSO TAKES THE MAPS FROM THE ORIGINAL "interferometer
# coordinates" INTO "bs coordinates."  "bs coordinates" are defined
# in a memo on the ACE memo board at
# http://www.srl.caltech.edu/ACE/forum/topics/ACE/

# WARNING: DO NOT EDIT getSisThickness.c!!!  YOUR CHANGES WILL BE
# LOST WHEN THIS PROGRAM IS RUN!!!  edit c code in the file
# getSisThickness.src, and then run this program which will
# read the local databases and generate getSisThickness.c

##### get information from sisStacks.db database #####

print "parsing sisStacks.db database...\n";
open(SIS_STACKS_DB, "sisStacks.db") || die $!;
$telescopeIDsString = "";
$maxPartition = 0;
$maxDepth = 0;
while (<SIS_STACKS_DB>) {
    unless (/([A-Z])([0-9]):(\d+(\s\d+)*)/) {
	print "Error in sisStacks.db line format:\n";
	print "\t$_\n";
	exit;
    }
    ($teleID, $partition, $detectorsString) = ($1, $2, $3);
    unless ($telescopeIDsString =~ /$teleID/) { $telescopeIDsString .= $teleID;}
    if ($partition > $maxPartition) { $maxPartition = $partition; }
    $partionsDetectors{"$teleID,$partition"} = $detectorsString;
    @detectors = split(/\s+/, $detectorsString);
    push(@allDetectors, @detectors);
    $detectorCountInThisPartition = scalar(@detectors);
    if ($detectorCountInThisPartition > $maxDepth) {
	$maxDepth = $detectorCountInThisPartition;
    }
}
$val{"_DETECTOR-COUNT_"} = scalar(@allDetectors);
@telescopeIDs = sort(split(//, $telescopeIDsString));
$val{"_TELES_"} = scalar(@telescopeIDs);
$val{"_PARTS_"} = $maxPartition;
$val{"_MAX-DPTH_"} = $maxDepth;

@allDetectors = sort(numerically @allDetectors);
for ($i = 0; $i < @allDetectors; $i++) {
    $detector = $allDetectors[$i];
    $detector =~ s/^0+//;				# remove starting 0's
    if ($detector eq "") { $detector = "   0"; }
    while (length($detector) < 4) { $detector = " $detector"; }
    if (($i % 10) == 0) { $val{"_DETECTORS-ARRAY_"} .= "    "; }
    $val{"_DETECTORS-ARRAY_"} .= "$detector, ";
    if (($i % 10) == 9) { $val{"_DETECTORS-ARRAY_"} .= "\n"; }
}
$val{"_DETECTORS-ARRAY_"} =~ s/[\s\,]*$//;	# remove terminal \s and ","s

$temp = "";
for ($i = 0; $i < @telescopeIDs; $i++) {
    $teleID = $telescopeIDs[$i];
    # $temp .= "\t/* telescope $teleID */\n";
    $temp .= "    {\n";
    for ($j = 0; $j < $maxPartition; $j++) {
	$partition = $j + 1;
	@detectors = split(/\s+/, $partionsDetectors{"$teleID,$partition"});
	for ($k = 0; $k < @detectors; $k++) {
	    $detectors[$k] =~ s/^0+//;			# remove starting 0's
	    if ($detectors[$k] eq "") { $detectors[$k] = "   0"; }
	    while (length($detectors[$k]) < 4) {
		$detectors[$k] = " $detectors[$k]";	# add spaces
	    }
	}
	for ($k = 0; $k < $maxDepth; $k++) { push(@detectors, "  -1") }
	# $temp .= "\t    /* partition $partition */\n";
	$temp .= "\t{ ";
	for ($k = 0; $k < $maxDepth; $k++) { $temp .= shift(@detectors) . ", ";}
	$temp =~ s/[\s\,]*$//;			# remove terminal \s and ","s
	$temp .= " },\n";
    }
    $temp =~ s/[\s\,]*$//;                  # remove terminal \s and ","s
    $temp .= "\n";
    $temp .= "    },\n";
}
$temp =~ s/[\s\,]*$//;                  # remove terminal \s and ","s
$val{"_DETECTORS-BY-INSTRUMENT-POSITION-ARRAY_"} = $temp;
undef $temp;

##### get information from any one interferometerData file #####

$dataDir = "interferometerData";

print "parsing an $dataDir database to get x and y node info...\n";
open(INTERFER_DATA, "$dataDir/ace$allDetectors[0].map") || die $!;
while (<INTERFER_DATA>) {
    unless (/^\s*(-?[\d\.]+)\s+(-?[\d\.]+)\s+(-?[\d\.]+)\s+([EM])\s*$/) {
	print "Error in $dataDir/ace$allDetectors[0].map line format:\n";
	print "\t$_\n";
	exit;
    }
    ($x, $y, $dt, $extrapFlag) = ($1, $2, $3, $4);	# $dt is not used here

	# IMPORTANT!!!!
	# this is where the extrapolation flag map is taken from interferometer
	# map coordinates into "bs" coordinates.  it involves a rotation
	# of the interferometer map of 180 degrees.

    $x *= -1;	if ($x == 0) { $x = "0"; }
    $y *= -1;	if ($y == 0) { $y = "0"; }

    push(@x, $x);
    push(@y, $y);
    $eFlag{"$x,$y"} = $extrapFlag;
}

@x = sort(numerically @x);
for ($i = 0; $i < @x; $i++) {
    if ($xShort[$#xShort] != $x[$i]) { push(@xShort, $x[$i]); }
}
undef @x;
$val{"_X-NODE-COUNT_"} = scalar(@xShort);
for ($i = 0; $i < @xShort; $i++) {
    $x = $xShort[$i];
    while (length($x) < 5) { $x = " $x"; }
    if (($i % 7) == 0) { $val{"_X-NODES-ARRAY_"} .= "    "; }
    $val{"_X-NODES-ARRAY_"} .= "$x,";
    if (($i % 7) == 6) { $val{"_X-NODES-ARRAY_"} .= "\n"; }
}
$val{"_X-NODES-ARRAY_"} =~ s/[\s\,]*$//;      # remove terminal \s and ","s
$val{"_NODE-SEPARATION-IN-X_"} = $xShort[1] - $xShort[0];

@y = sort(numerically @y);
for ($i = 0; $i < @y; $i++) {
    if ($yShort[$#yShort] != $y[$i]) { push(@yShort, $y[$i]); }
}
undef @y;
$val{"_Y-NODE-COUNT_"} = scalar(@yShort);
for ($i = 0; $i < @yShort; $i++) {
    $y = $yShort[$i];
    while (length($y) < 5) { $y = " $y"; }
    if (($i % 7) == 0) { $val{"_Y-NODES-ARRAY_"} .= "    "; }
    $val{"_Y-NODES-ARRAY_"} .= "$y,";
    if (($i % 7) == 6) { $val{"_Y-NODES-ARRAY_"} .= "\n"; }
}
$val{"_Y-NODES-ARRAY_"} =~ s/[\s\,]*$//;      # remove terminal \s and ","s
$val{"_NODE-SEPARATION-IN-Y_"} = $yShort[1] - $yShort[0];

$temp = "";
for ($i = 0; $i < @xShort; $i++) {
    $x = $xShort[$i];
    # $temp .= "\t/* x = $x */\n";
    $temp .= "    {";
    for ($j = 0; $j < @yShort; $j++) {
	$y = $yShort[$j];
	$temp .= " ".$eFlag{"$x,$y"}.",";
    }
    $temp =~ s/[\s\,]*$//;                  # remove terminal \s and ","s
    $temp .= " },\n";
}
$temp =~ s/[\s\,]*$//;                  # remove terminal \s and ","s
$temp =~ s/E/1/g;			# logical trues
$temp =~ s/M/0/g;			# logical falses
$val{"_EXTRAP-FLAGS-ARRAY_"} = $temp;
undef $temp;
	    
##### now get info from ALL interferometerData files #####

$temp = "";
for ($n = 0; $n < @allDetectors; $n++) {
    $detector = $allDetectors[$n];
    if (($detector == 100) || ($detector == 1000)) {
	$detectorName = "p" . $detector;
    }
    else { $detectorName = "ace" . $detector; }

    print "parsing $dataDir/$detectorName.map database for dt info...\n";
    open(MAP, "$dataDir/$detectorName.map") || die $!;
    undef %dt;
    while (<MAP>) {
	unless (/^\s*(-?[\d\.]+)\s+(-?[\d\.]+)\s+(-?[\d\.]+)\s+([EM])\s*$/) {
	    print "Error in $dataDir/$detectorName.map line format:\n";
	    print "\t$_\n";
	    exit;
	}
	($x, $y, $dt) = ($1, $2, $3);

	    # IMPORTANT STUFF HERE!!!!
	    # this is where corrections and changes are made to the original
	    # interferometer databases provided by mew.

	    # here we take the interferometer coordinates into "bs" coordinates.
	    # it involves rotating the interferometer map 180 degrees.  this
	    # change is also made above for the extrapolation flag array.

	$x *= -1;	if ($x == 0) { $x = "0"; }
	$y *= -1;	if ($y == 0) { $y = "0"; }

	    # AND HERE'S THAT DAMNED FACTOR OF 2!!! :)
	    # this is due to interferometer maps being off by a factor of 2.

	$dt{"$x,$y"} = $dt * 2;

    }
    # $temp .= "\t/* detector $detectorName */\n";
    $temp .= "    {\n";
    
    for ($i = 0; $i < @xShort; $i++) {
	$x = $xShort[$i];
	# $temp .= "\t    /* x = $x */\n";
	$temp .= "\t{ ";
	for ($j = 0; $j < @yShort; $j++) {
	    $y = $yShort[$j];
	    $dt = $dt{"$x,$y"};
	    while (length($dt) < 7) { $dt = " " . $dt; }
	    $temp .= "$dt,";
	    if (($j % 7) == 6) { $temp .= "\n\t  "; }
	}
	$temp =~ s/[\s\,]*$//;                  # remove terminal \s and ","s
	$temp .= "   },\n";
    }
    $temp =~ s/[\s\,]*$//;                  # remove terminal \s and ","s
    $temp .= "\n    },\n";
}
$temp =~ s/[\s\,]*$//;                  # remove terminal \s and ","s
#$temp .= "\n}";
$val{"_DT-ARRAY_"} = $temp;
undef $temp;

$program = "getSisThickness";
print "openning $program.src to read...\n";
open(TEMPLATE, "$program.src") || die $!;
if (-e "$program.c") {
    print "old version of $program.c exists.  save it? (y/n) [n] ";
    $answer = <STDIN>;
	if ($answer =~ /y/i) {
		$time = time;
		print "renaming old $program.c to $program.$time...\n";
		rename("$program.c", "$program.$time") || die $!;
	}
	else {
		print "removing old $program.c file...\n";
    	unlink("$program.c") || die $!;
	}
}
print "creating new $program.c file to write into...\n";
open(C, ">$program.c") || die $!;

print "printing edit warning...\n";
print C <<"STOP_PRINT";
/*
	WARNING!!!         WARNING!!!        WARNING!!!         WARNING!!! 

	WARNING: DO NOT EDIT THIS FILE!!!  IT IS AUTOMATICALLY GENERATED
	SOURCE CODE.  ANY CHANGES YOU MAKE WILL BE LOST WHEN THE PROGRAM
	generateGetSisThickness.pl IS NEXT RUN!  See the comments at the
	top of the generateGetSisThickness.pl program for more details.
*/

STOP_PRINT

print "printing $program.src into $program.c...\n";
while ($templateInput = <TEMPLATE>) {
    unless ($templateInput =~ /\b_\w+(-\w+)*_\b/) {
	print C $templateInput;
	next;
    }

    while ($templateInput =~ /\b_\w+(-\w+)*_\b/) {
	$tag = $&;				# tag = what was matched
	if ($tag eq "_TAG-WORDS-GO-HERE_") { last; }

	print "replacing tag $tag...\n";
	$templateInput =~ s/$tag/$val{$tag}/;
    }
    print C $templateInput;
}
print "\n\tjob finished.\n\n";

sub numerically { $a <=> $b; }
