#!/usr/local/bin/perl

# this program generates the c src code for readBaseThickness.c
# using the template file readBaseThickness.src which
# contains all the c code, plus many tags which will be replaced by this
# program.  it is done this way because the tags will be replaced
# with a large quantitiy of information taken from the databases affiliated
# with the readBaseThickness program.  the databases are much more
# maintainable in their current form than they would be if they were only
# held in the c source file.  thus the separation and generation.

# WARNING: DO NOT EDIT readBaseThickness.c!!!  YOUR CHANGES WILL BE
# LOST WHEN THIS PROGRAM IS RUN!!!  edit c code in the file
# readBaseThickness.src, and then run this program which will
# read the local databases and generate readBaseThickness.c

##### get information from absThickAt00.db database #####

print "parsing absThickAt00.db database...\n";
open(ABS_THICK_AT_00, "absThickAt00.db") || die $!;
$line = 0;
while (<ABS_THICK_AT_00>) {
    $line++;
    unless (/^NEW_RUN/) { next; }
    $_ = <ABS_THICK_AT_00>;
    unless (/^(\d+)\s+(\d+)\s+([\d\.]+)\s*$/) {
	print "Error in absThickAt00.db line format:\n";
	print "$line:$_\n";
	exit;
    }
    ($run, $detector, $thick) = ($1, $2, $3);
    $detector =~ s/^0+//;
    if ($detector eq "" ) { $detector = "0"; }
    while (length($detector) < 5) { $detector = " $detector"; }

	# IMPORTANT!!!  multiple values of the center thickness are
	# AVERAGED here.  affects detectors p1000, ace008, ace208
	# msu96 full maps of these detectors compared to interferometer
	# maps to get center thickness should be looked at in more
	# detail in order to determine if there is a better method
	# than just averaging the 2 results for each detector...

    if ($thickness{$detector}) {
	#$thickness{$detector} =~ s/^\s+//;
	$thickness{$detector} = ($thickness{$detector} + $thick) / 2;
    }
    else {
	$thickness{$detector} = $thick;
	push(@detectors, $detector);
    }
    while (length($thickness{$detector}) < 8) {
	$thickness{$detector} = " " . $thickness{$detector};
    }
}
$val{"_DETECTOR-COUNT_"} = scalar(@detectors);

@detectors = sort(numerically @detectors);
for ($i = 0; $i < @detectors; $i++) {
    $detector = $detectors[$i];
    $thickness = $thickness{$detector};
    if (($i % 7) == 0) {
	$val{"_DETECTORS-ARRAY_"} .= "\t";
	$val{"_THICKNESSES-AT-ORIGIN-ARRAY_"} .= "\t";
    }
    $val{"_DETECTORS-ARRAY_"} .= "$detector,";
    $val{"_THICKNESSES-AT-ORIGIN-ARRAY_"} .= "$thickness,";
    if (($i % 7) == 6) {
	$val{"_DETECTORS-ARRAY_"} .= "\n";
	$val{"_THICKNESSES-AT-ORIGIN-ARRAY_"} .= "\n";
    }
}
$val{"_DETECTORS-ARRAY_"} =~ s/[\s\,]*$//;	# remove terminal \s and ","s
$val{"_THICKNESSES-AT-ORIGIN-ARRAY_"} =~ s/[\s\,]*$//;

$program = "readBaseThickness";
print "openning $program.src to read...\n";
open(TEMPLATE, "$program.src") || die $!;
if (-e "$program.c") {
    print "old version of $program.c exists.  save it? (y/n) [n] ";
    $answer = <STDIN>;
	if ($answer =~ /y/i) {
		$time = time;
		print "renaming old $program.c to $program.$time...\n";
		rename("$program.c", "$program.$time") || die $!;
	}
	else {
		print "removing old $program.c file...\n";
    	unlink("$program.c") || die $!;
	}
}
print "creating new $program.c file to write into...\n";
open(C, ">$program.c") || die $!;

print "printing edit warning...\n";
print C <<"STOP_PRINT";
/*
	WARNING!!!         WARNING!!!        WARNING!!!         WARNING!!! 

	WARNING: DO NOT EDIT THIS FILE!!!  IT IS AUTOMATICALLY GENERATED
	SOURCE CODE.  ANY CHANGES YOU MAKE WILL BE LOST WHEN THE PROGRAM
	generateReadBaseThickness.pl IS NEXT RUN!  See the comments at the
	top of the generateReadBaseThickness.pl program for more details.
*/

STOP_PRINT

print "printing $program.src into $program.c...\n";
while ($templateInput = <TEMPLATE>) {
    unless ($templateInput =~ /\b_\w+(-\w+)*_\b/) {
	print C $templateInput;
	next;
    }

    while ($templateInput =~ /\b_\w+(-\w+)*_\b/) {
	$tag = $&;				# tag = what was matched
	if ($tag eq "_TAG-WORDS-GO-HERE_") { last; }

	print "replacing tag $tag...\n";
	$templateInput =~ s/$tag/$val{$tag}/;
    }
    print C $templateInput;
}
print "\n\tjob finished.\n\n";

sub numerically { $a <=> $b; }
