#include <stdio.h>

void parseStackDetectorID(char* stackDetectorID, int* detectorLocationInfo) {
    static const int MAX_PARTITION_DEPTH[] = { 0, 1, 1, 1, 1, 1, 3, 6, 1 };
 
/*
    stack detectors will be referred to as
    Tn[-m]L where   n = stack partition (1 - 8),
                    m = depth in partition if needed (starts at 1)
                    L = telescope ID "A" or "B"
    examples:
        T6-1B, T3A, T7-5A
*/

    int detectorIDFormatError = 0;
    int length, partitionOffset, partitionDepthOffset, telescopeOffset;
    int partition, partitionDepth, telescope;
 
    length = strlen(stackDetectorID);
    partitionOffset = 1;                /* same in both forms */
    if (length == 3) {
        partitionDepthOffset = -1;      /* missing in this form */
        telescopeOffset = 2;
    }
    else if (length == 5) {
        partitionDepthOffset = 3;
        telescopeOffset = 4;
    }
    else detectorIDFormatError = 1;
 
    if ((stackDetectorID[telescopeOffset] == 'A') ||
                (stackDetectorID[telescopeOffset] == 'a')) telescope = 1;
    else if ((stackDetectorID[telescopeOffset] == 'B') ||
                (stackDetectorID[telescopeOffset] == 'b')) telescope = 2;
    else detectorIDFormatError = 1;
 
    partition = stackDetectorID[partitionOffset] - '0';
    if ((partition < 1) || (partition > 8)) detectorIDFormatError = 1;
 
    if (partitionDepthOffset == -1) {
	    /* TnL format invalid for partitions 6 and 7 */
	if ((partition == 6) || (partition == 7)) detectorIDFormatError = 1;
	else partitionDepth = 1;
    }
    else {
	    /* Tn-mL format valid only for partitions 6 and 7 */
	if ((partition == 6) || (partition == 7)) {
            partitionDepth = stackDetectorID[partitionDepthOffset] - '0';
	}
	else detectorIDFormatError = 1;
    }
    if ((partitionDepth < 1) ||
	(partitionDepth > MAX_PARTITION_DEPTH[partition]))
						detectorIDFormatError = 1;
 
    if (detectorIDFormatError) {
        fprintf(stderr, "getInterferometerDt.parseStackDetectorID:\n");
        fprintf(stderr, "\tInvalid stack detector ID: [%s]\n", stackDetectorID);
        exit(0);
    }
 
    detectorLocationInfo[0] = partition;
    detectorLocationInfo[1] = partitionDepth;
    detectorLocationInfo[2] = telescope;
}
